/**
  \file Firmware.ino
  Copyright (C) 2018 Xoán Sampaíño

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <EscornabotEXT.h>
#include <EXTpad.h>

#define FW_VERSION "0.5.0"

const float WHEEL_DIAMETER = 80.0;
const float WHEEL_DISTANCE = 80.0;

const byte RIGHT_PINS[] = {2, 3, 4, 5};
const byte LEFT_PINS[] = {6, 7, 8, 9};

const byte BUTTON_PIN = A0;

const int GRID_SIZE = 100;
const int GRID_ANGLE = 90;

const int MOVE_LIMIT = 128;
const int PAUSE_MILLIS = 1000;

const int NOTE_A6 = 1760;

enum Move : byte {
  MOVE_NONE = 0,
  MOVE_FORWARD,
  MOVE_BACK,
  MOVE_LEFT,
  MOVE_RIGHT
};

/**
  \param wheel_diameter
  \param wheel_distance
  \param right_pins
  \param left_pins
*/
EscornabotEXT kanoi(WHEEL_DIAMETER, WHEEL_DISTANCE, RIGHT_PINS, LEFT_PINS);

/**
  \param button_pin
  \param none_value
  \param forward_value
  \param back_value
  \param left_value
  \param right_value
  \param go_value
*/
EXTpad keypad(BUTTON_PIN, 0, 672, 335, 501, 168, 846);

void setup() {
  kanoi.begin();
  keypad.begin();

  kanoi.beep(100, NOTE_A6);
}

void loop() {
  switch (keypad.scan()) {
    case BUTTON_FORWARD:
      record(MOVE_FORWARD);
      break;

    case BUTTON_FORWARD_LONG:
      break;

    case BUTTON_BACK:
      record(MOVE_BACK);
      break;

    case BUTTON_BACK_LONG:
      record(MOVE_NONE);
      break;

    case BUTTON_LEFT:
      record(MOVE_LEFT);
      break;

    case BUTTON_LEFT_LONG:
      break;

    case BUTTON_RIGHT:
      record(MOVE_RIGHT);
      break;

    case BUTTON_RIGHT_LONG:
      break;

    case BUTTON_GO:
      play();
      break;

    case BUTTON_GO_LONG:
      reset();
      break;
  }

  delay(10);
}

byte move_list[MOVE_LIMIT];
byte move_count = 0;

void record(byte move) {
  if (move_count < MOVE_LIMIT) {
    move_list[move_count] = move;
    move_count++;

    if (move_count == MOVE_LIMIT) {
      play();
    } else {
      kanoi.beep(100);
    }
  }
}

void reset() {
  if (move_count > 0) {
    move_count = 0;
  }

  kanoi.beep(100);
}

void play() {
  kanoi.beep(100, NOTE_A6);

  if (move_count > 0) {
    kanoi.wait(150);

    for (int i = 0; i < move_count; i++) {
      kanoi.beep(50);

      switch (move_list[i]) {
        case MOVE_FORWARD:
          kanoi.forward(GRID_SIZE);
          break;

        case MOVE_BACK:
          kanoi.back(GRID_SIZE);
          break;

        case MOVE_LEFT:
          kanoi.left(GRID_ANGLE);
          break;

        case MOVE_RIGHT:
          kanoi.right(GRID_ANGLE);
          break;

        case MOVE_NONE:
          kanoi.wait(PAUSE_MILLIS);
          break;
      }

      if (keypad.scan() != BUTTON_NONE) {
        break;
      }
    }

    move_count = 0;

    kanoi.beep(100, NOTE_A6);
  }

  kanoi.wait(100);
  kanoi.beep(150);
}

// EOF
